@@ -14,19 +14,6 @@ __email__ = "shyue@mit.edu"
 __status__ = "Production"
 __date__ ="Sep 23, 2011"
 
-def singleton(cls):
-    """
-    This decorator can be used to create a singleton out of a class.
-    """
-    
-    instances = {}
-    
-    def getinstance():
-        if cls not in instances:
-            instances[cls] = cls()
-        return instances[cls]
-    return getinstance
-
 class Enum(set):
     """
     Creates an enum out of a set.
