@@ -18,14 +18,14 @@ import json
 import itertools
 import warnings
 import numpy as np
-import copy
 
 from pymatgen.transformations.transformation_abc import AbstractTransformation
 from pymatgen.core.structure import Structure
+from pymatgen.core.lattice import Lattice
 from pymatgen.core.operations import SymmOp
 from pymatgen.core.structure_modifier import StructureEditor, SupercellMaker
 from pymatgen.core.periodic_table import smart_element_or_specie
-from pymatgen.analysis.ewald import EwaldSummation
+from pymatgen.analysis.ewald import EwaldSummation, minimize_matrix
 
 class IdentityTransformation(AbstractTransformation):
     """
@@ -263,94 +263,6 @@ class PartialRemoveSpecieTransformation2(AbstractTransformation):
         self._specie = specie_to_remove
         self._frac = fraction_to_remove
         
-    def _minimize_matrix(self, matrix, indices, num_to_remove, CURRENT_MINIMUM): #for whatever reason a default value wont work for CURRENT_MINIMUM
-                                                                            #must pass [float('inf')] VERY IMPORTANT
-                                                                            #current minimum works like a global variable since it is a list
-        '''minimize a matrix by removing a specific number of rows and columns (if row 4 is removed, column 4 must also be removed)
-        This method looks for short circuits to a brute force search by looking at best and worse case scenarios (which can be computed quickly)
-        It is about 1000 times faster than brute force'''
-        
-        if num_to_remove>len(indices):  #if we've kept too many rows
-            return[float('inf'), []]    #abandon the branch
-        
-        if num_to_remove==0:                        #if we don't have to remove any more rows
-            matrix_sum = sum(sum(matrix))
-            if matrix_sum<CURRENT_MINIMUM[0]:
-                CURRENT_MINIMUM[0] = matrix_sum
-            return [matrix_sum, []]                 #return the sum of the matrix
-        
-        indices = list(indices)         #make a copy of the indices so recursion doesn't alter them
-        matrix2=copy.deepcopy(matrix)
-        
-        max_index = None
-        
-        #compute the best case sum for removing rows
-        if num_to_remove>1: #lets not do this if we're finding the minimum in the next step anyway
-            index_sum = [sum(matrix[i,:])+sum(matrix[:,i])-matrix[i,i] for i in indices]    #compute the value associated with an index assuming no other indices are removed
-            max_index = indices[index_sum.index(max(index_sum))]                            #get the index of the maximum value (we'll try removing this row first)
-            index_sum_sorted = list(index_sum)
-            index_sum_sorted.sort(reverse = True)
-            all_interactions = list(matrix[indices,:][:,indices].flatten())                 #get the cells that we could be double counting when removing multiple index
-            all_interactions.sort()
-            
-            #sum the matrix - the rows with maximum 
-            best_case = sum(sum(matrix))-sum(index_sum_sorted[:num_to_remove])+sum(all_interactions[:(num_to_remove*(num_to_remove-1))])
-            
-            if best_case > CURRENT_MINIMUM[0]:
-                return [float('inf'), []]   #if the best case doesn't beat the minimum abandon the branch
-            
-            #try to find rows that should definitely be removed or definitely ignored based on best and worse case performances
-            most_positive = []
-            most_negative = []
-            for i in range(len(indices)):
-                index = indices[i]
-                interactions = [matrix[index,x]+matrix[x,index] for x in indices if not x == index]
-                interactions.sort()
-                most_positive.append(index_sum[i]-sum(interactions[:(num_to_remove-1)]))
-                most_negative.append(index_sum[i]-sum(interactions[-(num_to_remove-1):]))
-                
-            most_positive_sorted = sorted(most_positive, reverse = True)
-            most_negative_sorted = sorted(most_negative, reverse = True)    
-            
-            deletion_indices = []
-            ignore_indices = []
-            for i in range(len(indices)):
-                if most_negative[i] > most_positive_sorted[num_to_remove-1]:
-                    deletion_indices.append(indices[i])
-                    pass
-                if most_positive[i] < most_negative_sorted[num_to_remove-1]:
-                    ignore_indices.append(indices[i])
-                    pass
-                    
-            if deletion_indices + ignore_indices:
-                for r_index in deletion_indices:
-                    matrix2[:,r_index] = 0
-                    matrix2[r_index,:] = 0
-                    num_to_remove -= 1
-                    indices.remove(r_index)
-                for x in ignore_indices:
-                    indices.remove(x)
-                output = self._minimize_matrix(matrix2, indices, num_to_remove, CURRENT_MINIMUM)
-                output[1] = output[1]+deletion_indices
-                output[1].sort()
-                return output
-        
-        #if no shortcuts could be found, recurse down one level by both removing and ignoring one row
-        if max_index:
-            r_index = max_index
-            indices.remove(max_index)
-        else:    
-            r_index = indices.pop()
-        matrix2[:,r_index] = 0
-        matrix2[r_index,:] = 0
-        sum2 = self._minimize_matrix(matrix2, indices, num_to_remove-1, CURRENT_MINIMUM)
-        sum1 = self._minimize_matrix(matrix, indices, num_to_remove, CURRENT_MINIMUM)
-        if sum1[0]<sum2[0]:
-            return sum1
-        else:
-            sum2[1].append(r_index)
-            sum2[1].sort()
-            return sum2
     
     def apply_transformation(self, structure):
         sp = smart_element_or_specie(self._specie)
@@ -364,7 +276,7 @@ class PartialRemoveSpecieTransformation2(AbstractTransformation):
         
         ewaldmatrix = EwaldSummation(structure).total_energy_matrix
         
-        lowestenergy_indices = self._minimize_matrix(ewaldmatrix, specie_indices, num_to_remove, [float('inf')])[1]
+        lowestenergy_indices = minimize_matrix(ewaldmatrix, specie_indices, num_to_remove)[1]
         
         mod = StructureEditor(structure)
         mod.delete_sites(lowestenergy_indices)
@@ -496,6 +408,134 @@ class OrderDisorderedStructureTransformation(AbstractTransformation):
         output['init_args'] = {}
         return output
 
+class PrimitiveCellTransformation(AbstractTransformation):
+    """
+    This class finds the primitive cell of the input structure. 
+    It returns a structure that is not necessarily orthogonalized
+    Author: Will Richards
+    """
+    def __init__(self, tolerance = 0.2):
+        self._tolerance = tolerance
+    
+    def _get_more_primitive_structure(self, structure, tolerance):
+        '''this finds a smaller unit cell than the input
+        sometimes it doesn't find the smallest possible one, so this method is called until it
+        is unable to find a smaller cell
+        
+        The method works by finding transformational symmetries for all sites and then using
+        that translational symmetry instead of one of the lattice basis vectors
+        if more than one vector is found (usually the case for large cells) the one with the 
+        smallest norm is used
+        
+        Things are done in fractional coordinates because its easier
+        to translate back to the unit cell'''
+        
+        #convert tolerance to fractional coordinates
+        tol_a = tolerance/structure.lattice.a
+        tol_b = tolerance/structure.lattice.b
+        tol_c = tolerance/structure.lattice.c
+        
+        #get the possible symmetry vectors
+        sites = sorted(structure.sites, key = lambda site: site.species_string)
+        grouped_sites = [list(group) for k, group in itertools.groupby(sites, key = lambda site: site.species_string)]
+        min_site_list = min(grouped_sites, key = lambda group: len(group))
+                
+        x = min_site_list[0]
+        possible_vectors = []
+        for y in min_site_list:
+            if not x == y:
+                vector = (x.frac_coords - y.frac_coords)%1
+                possible_vectors.append(vector)
+                
+        #test each vector to make sure its a viable vector for all sites
+        for x in sites:
+            for j in range(len(possible_vectors)):
+                p_v = possible_vectors[j]
+                fit = False
+                if p_v is not None: #have to test that adding vector to a site finds a similar site
+                    test_location = x.frac_coords + p_v
+                    possible_locations = [site.frac_coords for site in sites if site.species_and_occu == x.species_and_occu and not x == site]
+                    for p_l in possible_locations:
+                        diff = .5 - abs((test_location-p_l)%1 -.5)
+                        if diff[0] < tol_a and diff[1] < tol_b and diff[2] < tol_c:
+                            fit = True
+                            break
+                    if not fit:
+                        possible_vectors[j] = None
+        
+        #vectors that haven't been removed from possible_vectors are symmetry vectors
+        #convert these to the shortest representation of the vector           
+        symmetry_vectors = [.5-abs((x-.5)%1) for x in possible_vectors if x is not None]
+        if symmetry_vectors:
+            reduction_vector = min(symmetry_vectors, key = np.linalg.norm)
+            
+            #choose a basis to replace (a, b, or c)
+            proj = abs(structure.lattice.abc * reduction_vector)
+            basis_to_replace = list(proj).index(max(proj))
+            
+            #create a new basis
+            new_matrix = structure.lattice.matrix
+            new_basis_vector = np.dot(reduction_vector, new_matrix)
+            new_matrix[basis_to_replace] = new_basis_vector
+            new_lattice = Lattice(new_matrix)
+            
+            #create a structure with the new lattice
+            new_structure = Structure(new_lattice, structure.species_and_occu,
+                                      structure.cart_coords, coords_are_cartesian = True)
+            
+            #update sites and tolerances for new structure
+            sites = list(new_structure.sites)
+        
+            tol_a = tolerance/new_structure.lattice.a
+            tol_b = tolerance/new_structure.lattice.b
+            tol_c = tolerance/new_structure.lattice.c
+            
+            #Make list of unique sites in new structure
+            new_sites = []
+            for site in sites:
+                fit = False
+                for new_site in new_sites:
+                    if site.species_and_occu == new_site.species_and_occu:
+                        diff = .5 - abs((site.frac_coords - new_site.frac_coords)%1-.5)
+                        if diff[0] < tol_a and diff[1] < tol_b and diff[2] < tol_c:
+                            fit = True
+                            break
+                if not fit:
+                    new_sites.append(site)
+            
+            #recreate the structure with just these sites
+            new_structure = Structure(new_structure.lattice,[site.species_and_occu for site in new_sites],
+                                  [(site.frac_coords+.001)%1-.001 for site in new_sites])
+        
+            return new_structure
+        else: #if there were no translational symmetry vectors
+            return structure
+    
+    def apply_transformation(self, structure):
+        """
+        """
+        structure2 = self._get_more_primitive_structure(structure, self._tolerance)
+        while len(structure2)<len(structure):
+            structure = structure2
+            structure2 = self._get_more_primitive_structure(structure, self._tolerance)
+        return structure2
+        
+    
+    def __str__(self):
+        return "Primitive cell transformation"
+     
+    def __repr__(self):
+        return self.__str__()
+    
+    @property
+    def inverse(self):
+        return None
+    
+    @property
+    def to_dict(self):
+        output = {'name' : self.__class__.__name__}
+        output['init_args'] = {}
+        return output
 
 def transformation_from_json(json_string):
     """
